#ifndef code__validation__macros_hxx_
#define code__validation__macros_hxx_

#include <code/validation/assert.hxx>
#include <code/validation/setup.hxx>
#include <code/validation/teardown.hxx>
#include <code/validation/test-case.hxx>

#define VALIDATION_TEST_SETUP_2(file, line)                                                \
        static void test_setup_func_##line();                                              \
        code::validation::Setup test_setup_##line(file, line, &test_setup_func_##line); \
        void test_setup_func_##line()

#define VALIDATION_TEST_SETUP_1(file, line) \
        VALIDATION_TEST_SETUP_2(file, line)

#define VALIDATION_TEST_SETUP \
        VALIDATION_TEST_SETUP_1(__FILE__, __LINE__)

#define VALIDATION_TEST_TEARDOWN_2(file, line)                                                      \
        static void test_teardown_func_##line();                                                    \
        code::validation::Teardown test_teardown_##line(file, line, &test_teardown_func_##line); \
        void test_teardown_func_##line()

#define VALIDATION_TEST_TEARDOWN_1(file, line) \
        VALIDATION_TEST_TEARDOWN_2(file, line)

#define VALIDATION_TEST_TEARDOWN \
        VALIDATION_TEST_TEARDOWN_1(__FILE__, __LINE__)

#define VALIDATION_TEST_2(name, file, line)                                                   \
        void test_func_##line();                                                              \
        code::validation::Test_case test_##line(#name, (file), (line), &test_func_##line); \
        void test_func_##line()

#define VALIDATION_TEST_1(name, file, line) \
        VALIDATION_TEST_2(name, file, line)

#define VALIDATION_TEST(name) \
        VALIDATION_TEST_1(name, __FILE__, __LINE__)

#define VALIDATION_ASSERT_TRUE(expr) \
        ::code::validation::Assert::assert_true((expr), #expr)

#define VALIDATION_ASSERT_FALSE(expr) \
        ::code::validation::Assert::assert_false((expr), #expr)

#define VALIDATION_ASSERT_NULL(expr) \
        ::code::validation::Assert::assert_null((expr), #expr)

#define VALIDATION_ASSERT_EQUAL(lhs, rhs) \
        ::code::validation::Assert::assert_equal((lhs), (rhs), #lhs, #rhs)

#endif
